@@ -24,8 +24,8 @@ function exec_mutualisation_dist() {
 	$version_spip = intval($GLOBALS['spip_version_branche']) ;
 
 
-	$url_stats = "ecrire/?exec=statistiques_visites";
-	$url_compresseur = "ecrire/?exec=config_fonctions#configurer-compresseur";
+	$url_stats = "ecrire/?exec=stats_visites";
+	$url_compresseur = "ecrire/?exec=configurer_avancees#formulaire_configurer_compresseur";
 	$url_admin_plugin = "ecrire/?exec=admin_plugin";
 
 	if (!file_exists(_DIR_IMG.'mutualiser.png'))
@@ -94,6 +94,9 @@ function exec_mutualisation_dist() {
 				foreach ($plugins as $plugin) {
 					$lsplugs[strtolower($plugin)][] = $alias[$v];
 					$versionplug[strtolower($plugin)] = $cfg[$plugin]['version'];
+					// Spip n'est pas un plugin… Mais en fait oui.
+					// unset($lsplugs['spip']);
+					// unset($versionplug['spip']);
 				}
 				$cntplugins = count($plugins);
 				$plugins = strtolower(join(', ', $plugins));
@@ -106,7 +109,6 @@ function exec_mutualisation_dist() {
 			// va intercepter (pas terrible ?)
 			$erreur = test_upgrade_site($meta);
 			$adminplugin = adminplugin_site($meta, $liste_plug_compat, $liste_plug_compat_base);
-			$credential = test_credential_site($meta, $v);
 			$version_installee = ' <em><small>'.$meta['version_installee'].'</small></em>';
 		}
 		else {
@@ -114,7 +116,7 @@ function exec_mutualisation_dist() {
 			$erreur = ' <em><small><span class="erreur">Erreur&nbsp;!</span></small></em>';
 			$plugins = '-';
 		}
-		
+
 		$compression = '';
 		if ($meta['auto_compress_css']=='oui')
 			$compression .= 'CSS';
@@ -128,82 +130,119 @@ function exec_mutualisation_dist() {
 		//<![CDATA[
 		tableau_sites.push(["../../'.$GLOBALS['mutualisation_dir'].'/'.$v.'"]);
 		//]]>
-		</script>';
+		</script>
+		';
 
 		$page .= "<tr class='tr". $nsite % 2 ."'"
-			. " style='background-image: url(${url}ecrire/index.php?exec=mutualisation&amp;renouvelle_alea=yo)' id='$alias[$v]'>
-			<td style='text-align:right;'><img src='${url}favicon.ico' style='float:left;' />$v$erreur$credential$version_installee</td>
-			<td><a href='${url}'>".typo($nom_site)."</a></td>
-			<td><a href='${url}ecrire/'>ecrire</a></td>
-			<td><div id='IMG$nsite' class='taille loading'></div></td>
-			<td><div id='local$nsite' class='taille loading'></div></td>
-			<td><div id='cache$nsite' class='taille loading'></div></td>
-			<td style='text-align:right;'><a href='${url}$url_stats'>${stats}</a></td>
-			<td>$adminplugin<a href='${url}$url_admin_plugin'>${cntplugins}</a> <small>${plugins}</small></td>
-			<td><a href='${url}$url_compresseur'>$compression</a></td>
-			<td style='text-align:right;'>".date_creation_repertoire_site($v)."</td>
+			. " style='background-image: url(${url}ecrire/index.php?exec=mutualisation&amp;renouvelle_alea=yo)' id='$alias[$v]'>\n
+			<td style='text-align:right;'><img src='${url}favicon.ico' style='float:left;' />$v$erreur$version_installee</td>\n
+			<td><a href='${url}'>".typo($nom_site)."</a></td>\n
+			<td><a href='${url}ecrire/'>ecrire</a></td>\n
+			<td><div id='IMG$nsite' class='taille loading'></div></td>\n
+			<td><div id='local$nsite' class='taille loading'></div></td>\n
+			<td><div id='cache$nsite' class='taille loading'></div></td>\n
+			<td style='text-align:right;'><a href='${url}$url_stats'>${stats}</a></td>\n
+			<td>$adminplugin<a href='${url}$url_admin_plugin'>${cntplugins}</a> <small>${plugins}</small></td>\n
+			<td><a href='${url}$url_compresseur'>$compression</a></td>\n
+			<td style='text-align:right;'>".date_creation_repertoire_site($v)."</td>\n
 			</tr>\n";
 		$nsite++;
 	}
 	$page .= "</tbody></table>";
 
+	// On liste ici tous les plugins-dist de la mutu.
+	// Ça sera calculé une seule fois pour toute et réutilisé dans la fonction plus loin.
+	$list_dist = array();
+	// correspond à plugins-dist/nom_plugin/paquet.xml
+	if (glob(_DIR_PLUGINS_DIST . '*/paquet.xml',GLOB_NOSORT)) {
+		foreach (glob(_DIR_PLUGINS_DIST . '*/paquet.xml',GLOB_NOSORT) as $value) {
+			if (preg_match('/prefix="([^"]*)"/i', file_get_contents($value), $r)){
+					$list_dist[] = strtolower(trim($r[1]));
+				}
+		}
+	}
+
 
 	if ($lsplugs) {
 		$nombre_plugins = count($lsplugs) ;
 		$page .= "<br /><br /><table style='clear:both;'>
-	<thead>
+	<thead>\n
 		<tr>
 			<td>#</td>
 			<td>Plugins utilis&#233;s ($nombre_plugins) </td>
+			<td>Plugins-dist</td>
 			<td>Version</td>
 			<td>Sites</td>
-		</tr>
+		</tr>\n
 	</thead>
 	<tbody>";
 		foreach ($lsplugs as $plugin => $c){
-			$plnum[count($c)] .= "<tr><td>".count($c)."</td><td>$plugin</td>"
-				."<td>".$versionplug[$plugin]."</td><td>".join(', ', ancre_site($c)).'</td></tr>';
+			$plnum[count($c)] .= "<tr>\n<td>".count($c)."</td>\n<td>$plugin</td>\n"
+				."<td>" . pluginDist($list_dist,$plugin) ."</td>\n<td>".$versionplug[$plugin]."</td>\n<td>".join(', ', ancre_site($c)).'</td>' . "\n" .'</tr>'. "\n";
 		}
 		krsort($plnum);
 		$page .= join('', $plnum);
-		$page .= "</tbody></table>\n";
+		$page .= "</tbody>\n</table>\n";
 
 
 		$inutile = array();
 		$extract = array();
 		$list = array();
+		// On crée une variable ici qui regardera les particularités des fichiers xml d'un plugin.
+		// Si à l'avenir on change de terminologie de fichier xml, il suffira de l'ajouter dans un nouvel array()
+		$cfg = array(
+			array(
+			'fn' => 'paquet.xml',
+			'pre' => '/prefix="([^"]*)"/i',
+			'ver' => '/version="([^"]*)"/i',
+			),
+			array(
+			'fn' => 'plugin.xml',
+			'pre' => ',<prefix>([^<]+),ims',
+			'ver' => ',<version>([^<]+),ims',
+			),
+		);
+
 		
 		$ustart_glob = memory_get_peak_usage(true);
-		// Si on est sur un spip 2
-		// correspond à plugins/nom_plugin/fichier.xml
-		if (glob(_DIR_PLUGINS . '*/plugin.xml')) {
-			foreach (glob(_DIR_PLUGINS . '*/plugin.xml') as $value) {
-				$list[] = $value;
-			}
-		}
-		// correspond à plugins/auto/nom_plugin/fichier.xml
-		if (glob(_DIR_PLUGINS . '*/*/plugin.xml')) {
-			foreach (glob(_DIR_PLUGINS . '*/*/plugin.xml') as $value) {
-				$list[] = $value;
-			}
-		}
+		// Ici on est en SPIP 3.
+		// En spip 3, avec SVP, on liste les plugins dans des sous-répertoires.
+		// Ca peut aller jusqu'a 3 sous-répertoires.
+		// On garde l'ancien principe d'un sous-répertoire pour ne pas casser la compat.
 
-		foreach ($list as $url) {
-			// là, on reprend l'ancien code. On cherche la valeur de la balise prefix
-			if (preg_match(',<prefix>([^<]+),ims', file_get_contents($url), $r)
-				AND !$lsplugs[strtolower(trim($r[1]))])
-					$inutile[] = trim($r[1]);
-		}
+		$dir = _DIR_PLUGINS;
+		$dir_it = new RecursiveDirectoryIterator($dir, FilesystemIterator::SKIP_DOTS);
+		$it = new RecursiveIteratorIterator($dir_it, RecursiveIteratorIterator::SELF_FIRST);
 
+		foreach($it as $path => $fo) {
+
+		  if ( !$fo->isDir() ) {
+		    continue;
+		  }
+
+		  $path .= '/';
+
+		  foreach ($cfg as $k => $v) {
+		    if ( file_exists($path . $v['fn']) ) {
+		      $res = processConfig($cfg[$k], $lsplugs, $path);
+		      if (false !== $res) {
+		      	$inutile[] = $res;
+		      }
+		      break;
+		    }
+		  }
+		}
 		$uend_glob = memory_get_peak_usage(true);
-		
+			
+
 		$inutile = array_map('mb_strtolower', $inutile);
 		sort($inutile);
 
+
 		if ($inutile) {
 			$nombre_plugins_inutiles =count($inutile) ;
-			$page .= "<p><strong>"._L('Plugins inutilis&#233;s :')."</strong> ".join(', ', $inutile).".<br />";
-			$page .= "<em>Soit " . $nombre_plugins_inutiles . _L(' plugins inutilis&#233;s') . ".</em></p>";
+			$page .= "<div class='inutilises'>\n<p><strong>"._L('Plugins inutilis&#233;s :')."</strong> ".join(', ', $inutile).".<br />";
+			$page .= "<em>Soit " . $nombre_plugins_inutiles . _L(' plugins inutilis&#233;s') . ".</em></p>\n</div>";
 		}
 	}
 
@@ -228,48 +267,47 @@ function exec_mutualisation_dist() {
 	$time=$timeend-$timestart;
 	$page_load_time = number_format($time, 3);
 
-	if (isset($_GET['debug'])) {
-		$debug_toolbar = "<div class='toolbar'>\n";
+	// On génère le contenu de notre toolbar.
+	$debug_toolbar = "<div class='toolbar'>\n";
 
-		$debug_toolbar .= "<div class='toolbar-block'>\n";
-		$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-php_info'></i></div>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info'>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>SPIP</b> <span>" . $GLOBALS['spip_version_branche'] . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>PHP</b> <span>" . phpversion() . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Mémoire allouées</b> <span>" . $memory_limit . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Serveur</b> <span>" . $_SERVER['SERVER_SOFTWARE'] . "</span></div>\n";
-		$debug_toolbar .= "</div></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-block'>\n";
+	$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-php_info'></i></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info'>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>SPIP</b> <span>" . $GLOBALS['spip_version_branche'] . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>PHP</b> <span>" . phpversion() . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Mémoire allouée</b> <span>" . $memory_limit . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Serveur</b> <span>" . $_SERVER["SERVER_SOFTWARE"] . "</span></div>\n";
+	$debug_toolbar .= "</div></div>\n" ;
 
-		$debug_toolbar .= "<div class='toolbar-block'>\n";
-		$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-plugins'></i><span>". ($nombre_plugins_inutiles + $nombre_plugins) ." plugins</span></div>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info'>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Utilisés</b> <span>" . $nombre_plugins . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Inutilisés</b> <span>" . $nombre_plugins_inutiles . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Total</b> <span>" . ($nombre_plugins_inutiles + $nombre_plugins) . "</span></div>\n";
-		$debug_toolbar .= "</div></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-block'>\n";
+	$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-plugins'></i><span>". ($nombre_plugins_inutiles + $nombre_plugins) ." plugins</span></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info'>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Utilisés</b> <span>" . $nombre_plugins . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Inutilisés</b> <span>" . $nombre_plugins_inutiles . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Total</b> <span>" . ($nombre_plugins_inutiles + $nombre_plugins) . "</span></div>\n";
+	$debug_toolbar .= "</div></div>\n" ;
 
-		$debug_toolbar .= "<div class='toolbar-block'>\n";
-		$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-memory'></i> <span>". memoryUsage($udiff) . "</span></div>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info'>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Mémoire :</b></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Au début</b> <span>" . memoryUsage($ustart) . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>À la fin</b> <span>" . memoryUsage($uend) . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Différence</b> <span>" . memoryUsage($udiff) . "</span></div>\n";
-		$debug_toolbar .= "</div></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-block'>\n";
+	$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-memory'></i> <span>". memoryUsage($udiff) . "</span></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info'>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Mémoire :</b></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Au début</b> <span>" . memoryUsage($ustart) . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>À la fin</b> <span>" . memoryUsage($uend) . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Différence</b> <span>" . memoryUsage($udiff) . "</span></div>\n";
+	$debug_toolbar .= "</div></div>\n" ;
 
-		$debug_toolbar .= "<div class='toolbar-block'>\n";
-		$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-time'></i> <span>". $page_load_time . " s</span></div>\n" ;
-		$debug_toolbar .= "<div class='toolbar-info'>" ;
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Début du script</b> <span>" . date("H:i:s", $timestart) . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Fin du script</b> <span>" . date("H:i:s", $timeend) . "</span></div>\n";
-		$debug_toolbar .= "<div class='toolbar-info-element'><b>Temps d'exécution</b> <span>" . $page_load_time . " s</span></div>\n";
-		$debug_toolbar .= "</div></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-block'>\n";
+	$debug_toolbar .= "<div class='toolbar-icon'><i class='icon-time'></i> <span>". $page_load_time . " s</span></div>\n" ;
+	$debug_toolbar .= "<div class='toolbar-info'>" ;
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Début du script</b> <span>" . date("H:i:s", $timestart) . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Fin du script</b> <span>" . date("H:i:s", $timeend) . "</span></div>\n";
+	$debug_toolbar .= "<div class='toolbar-info-element'><b>Temps d'exécution</b> <span>" . $page_load_time . " s</span></div>\n";
+	$debug_toolbar .= "</div></div>\n" ;
 
-		$debug_toolbar .= "</div>\n" ;
+	$debug_toolbar .= "</div>\n" ;
 
-		$page = str_replace('</body>', $debug_toolbar . "\n </body>", $page);
+	$page = str_replace('</body>', $debug_toolbar . "\n </body>", $page);
 
-	}
 	echo $page;
 }
 
@@ -337,30 +375,6 @@ function mutualisation_lister_sites_dist() {
 	return $sites;
 }
 
-// lister les sites qui ont des sites/xx/config/connect.php
-// avec 'connect.php' ne changeant pas de nom
-function test_credential_site($meta, $v) {
-	if (defined("_INSTALL_USER_DB_ROOT")) {
-		$file = file_get_contents('../'.$GLOBALS['mutualisation_dir'].'/'.$v.'/config/connect.php');
-		$credential = strpos($file, _INSTALL_USER_DB_ROOT);
-		if ($credential) {
-			$secret = $meta['version_installee'].'-'.$meta['popularite_total'];
-			$secret = md5($secret);
-			return <<<EOF
-<form action='$meta[adresse_site]/ecrire/index.php?exec=mutualisation' method='post' class='upgrade' target='_blank'>
-<div>
-<input type='hidden' name='secret' value='$secret' />
-<input type='hidden' name='exec' value='mutualisation' />
-<input type='hidden' name='credential' value='oui' />
-<input type='submit' value='Correct DB credential' />
-</div>
-</form>
-EOF;
-		}
-	}
-	return '';
-}
-
 /* autre exemple pour ceux qui mettent tous leurs fichiers de connexion
  * dans /config/connect/xx.php
  * fonction a mettre dans mes_options.php ou dans mutualisations/lister_sites.php
@@ -402,4 +416,28 @@ function memoryUsage($bytes) {
         return $bytes . ' B';
 }
 
-?>
+function pluginDist($array, $plugin) {
+
+	$p = "-";
+	if (in_array($plugin, $array)) $p = "Oui";
+	return $p;
+}
+
+/* Petite fonction qui va automatiser la recherche de paquet.xml ou plugin.xml 
+ * quelque soit la profondeur dans l'arborescence
+ */
+function processConfig(&$cfg, &$lsplugs, $path) {
+
+	// echo "<!-- Process: " . $path . $cfg['fn'] . "--> \n";
+
+	$data = file_get_contents($path . $cfg['fn']);
+
+	if ( 1 === preg_match($cfg['pre'], $data, $r) AND !$lsplugs[strtolower(trim($r[1]))] ) {
+		preg_match($cfg['ver'], $data, $n);
+		return trim($r[1]) . ' (' . $n[1] . ')';
+	}
+
+	return false;
+}
+
+?>
\ No newline at end of file
